import traceback

from enigma import eServiceReference
from Screens.InfoBarGenerics import (InfoBarShowHide, InfoBarSeek, InfoBarNotifications)
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import HelpableActionMap
from Components.ServiceEventTracker import InfoBarBase
from Tools.BoundFunction import boundFunction

class Player(object):
    def __init__(self, session, playerType=4097, callback=None):
        self.session = session
        self.prevService = session.nav.getCurrentlyPlayingServiceReference()
        self.callback = callback
        self.playerType = playerType
        self.player = None
    def play(self, title, playUrls):
        def selectStream(callback=None):
            if callback:
                runPlayer(callback[0], callback[1])
        def runPlayer(pTitle, url):
            service_ref = eServiceReference(self.playerType, 0, url)
            if self.player is None:
                self.player = self.session.openWithCallback(self.playerExit, MyPlayer, self.playerCallback)
            service_ref.setName(pTitle)
            self.player.setInfoBarText(pTitle)
            try:
                self.player.playRef(service_ref)
            except Exception as err:
                raise err
        print("play urls=\n%s"%playUrls)

        if isinstance(playUrls, list):
            if len(playUrls)==0:
                self.session.open(MessageBox, text="No video streams to play", type=MessageBox.TYPE_ERROR, timeout = 15)
            elif len(playUrls) == 1:
                runPlayer(playUrls[0]['title'], playUrls[0]['url'])
            else:
                choices = []
                for x in playUrls:
                    choices.append((x['title'], x['url']))
                self.session.openWithCallback(selectStream, ChoiceBox, title="Please select source", list=choices)
        elif isinstance(playUrls, str):
            runPlayer(title, playUrls)
        else:
            self.session.open(MessageBox, text="Invalid playUrls type", type=MessageBox.TYPE_ERROR, timeout = 15)

        
    def playerCallback(self, callback):
        try:
            self.session.nav.playService(self.prevService)
            if callback:
                self.player.close()
        except:
            self.playerExit()
    def playerExit(self):
        self.player = None
        if self.callback:
            try:
                self.callback()
            except:
                pass
            self.callback = None

class MyPlayer(InfoBarBase, InfoBarSeek, InfoBarNotifications, InfoBarShowHide, Screen):
    def __init__(self, session, callback):
        Screen.__init__(self, session)
        self.skinName = ["MoviePlayer"]
        InfoBarBase.__init__(self)
        InfoBarShowHide.__init__(self)
        InfoBarSeek.__init__(self)
        InfoBarNotifications.__init__(self)
        #HelpableScreen.__init__(self)
        self.player_callback = callback
        self["actions"] = HelpableActionMap(self, "MyActions",
                {
                "cancel":(boundFunction(self.player_callback, ("exit",)), "Exit player"),
                }, -2)
        self.onClose.append(self.__on_close)

    def playRef(self, service_ref):
        self.session.nav.stopService()
        self.session.nav.playService(service_ref)

    def doEofInternal(self, playing):
        self.player_callback(("eof", playing))

    def setInfoBarText(self, title):
        try:
            self.setTitle(title)
        except:
            print(traceback.format_exc())

    def __on_close(self):
        pass
        